'Revolved Feature Example

Private Sub cmdRevolvedFeatures_Click()
    Const Pi As Double = 3.14159265358979
    Dim objApp As Object
    Dim objDoc As Object
    Dim objRefPlane As Object
    Dim objProfile As Object
    Dim objProfiles(1 To 2) As Object
    Dim dblX As Double
    Dim dblY As Double
    Dim objAxisLine As Object
    Dim objRefAxis As Object
    Dim ordStatus As Long
    Dim objLines(1 To 4) As Object
    Dim objModel As Object
    
    
    ' Connect to a running instance of Solid Edge.

    Set objApp = GetObject(, "SolidEdge.Application")
    
    ' Create a new Part document.
    Set objDoc = objApp.Documents.Add(ProgID:="SolidEdge.PartDocument", _
                                    TemplateDoc:="Normal.par")
    
    ' Create a new reference plane.
    Set objRefPlane = objDoc.RefPlanes.AddParallelByDistance( _
                            ParentPlane:=objDoc.RefPlanes(3), _
                            distance:=0, _
                            NormalSide:=igLeft, _

                            Local:=True)
                            
    ' Create a new profile to use for the feature.
    Set objProfiles(1) = objDoc.ProfileSets.Add.Profiles.Add(objRefPlane)
    
    ' Get the model space origin relative to profile space.
    Call objProfiles(1).Convert3DCoordinate(x3d:=0,_
                                  y3d:=0, z3d:=0, _
                                  x2d:=dblX, y2d:=dblY)
    
    ' Draw a circle for the profile graphics.

    Call objProfiles(1).Circles2d.AddByCenterRadius(_
                             x:=dblX, y:=dblY + 0.1, _
                             Radius:=0.075)
    
    ' Draw a line to use for the axis of revolution.
    Set objAxisLine = objProfiles(1).Lines2d.AddBy2Points(_
                                      X1:=dblX, Y1:=dblY, _
                                      X2:=dblX + 0.1, Y2:=dblY)
    
    ' Use the line just created to create the axis of revolution.

    Set objRefAxis = objProfiles(1).SetAxisOfRevolution(LineForAxis:=objAxisLine)
    
    ' End and validate the profile.
    ordStatus = objProfiles(1).End(ValidationCriteria:=igProfileClosed + _
                                igProfileRefAxisRequired)
    If ordStatus <> 0 Then
        MsgBox "Invalid Profile"
        Exit Sub
    End If
    
   ' Create 360 degree revolved base feature.
    Set objModel = objDoc.Models.AddFiniteRevolvedProtrusion(NumberOfProfiles:=1, _

                            ProfileArray:=objProfiles, _
                            ReferenceAxis:=objRefAxis, _
                            ProfilePlaneSide:=igLeft, _
                            AngleOfRevolution:=Pi * 2)
    
    ' Turn off the display of the profile.
    objProfiles(1).Visible = False
    
    ' Create a new reference plane.
    Set objRefPlane = objDoc.RefPlanes.AddParallelByDistance( _
                            ParentPlane:=objDoc.RefPlanes(3), _

                            distance:=0, _
                            NormalSide:=igLeft, _
                            Local:=True)
                            
    ' Create a new profile to use for the feature.
    Set objProfile = objDoc.ProfileSets.Add.Profiles.Add(objRefPlane)
    
    ' Get the model space origin relative to profile space.
    Call objProfile.Convert3DCoordinate(x3d:=0, y3d:=0, z3d:=0, _
                                        x2d:=dblX, y2d:=dblY)


    ' Create three lines to use for the profile geometry.
    Set objLines(1) = objProfile.Lines2d.AddBy2Points(_
                         X1:=dblX - 0.02, Y1:=dblY + 0.2, _
                         X2:=dblX - 0.02, Y2:=dblY + 0.15)
    Set objLines(2) = objProfile.Lines2d.AddBy2Points(_
                         X1:=dblX - 0.02, Y1:=dblY + 0.15, _
                         X2:=dblX + 0.02, Y2:=dblY + 0.15)
    Set objLines(3) = objProfile.Lines2d.AddBy2Points(_

                         X1:=dblX + 0.02, Y1:=dblY + 0.15, _
                         X2:=dblX + 0.02, Y2:=dblY + 0.2)
    Set objLines(4) = objProfile.Lines2d.AddBy2Points(_
                         X1:=dblX + 0.02, Y1:=dblY + 0.2, _
                         X2:=dblX - 0.02, Y2:=dblY + 0.2)
    
    ' Connect the lines with relationships.
    Call objProfile.Relations2d.AddKeypoint(_
                            Object1:=objLines(1), _
                            Index1:=igLineEnd, _

                            Object2:=objLines(2), _
                            Index2:=igLineStart, _
                            guaranteed_ok:=True)
    Call objProfile.Relations2d.AddKeypoint(_
                      Object1:=objLines(2), _
                      Index1:=igLineEnd, _
                      Object2:=objLines(3), _
                      Index2:=igLineStart, _
                      guaranteed_ok:=True)
    Call objProfile.Relations2d.AddKeypoint(_

                      Object1:=objLines(3), _
                      Index1:=igLineEnd, _
                      Object2:=objLines(4), _
                      Index2:=igLineStart, _
                      guaranteed_ok:=True)
    Call objProfile.Relations2d.AddKeypoint(_
                      Object1:=objLines(4), _
                      Index1:=igLineEnd, _
                      Object2:=objLines(1), _
                      Index2:=igLineStart, _
                      guaranteed_ok:=True)

    
    ' Draw a line to use for the axis of revolution.
    Set objAxisLine = objProfile.Lines2d.AddBy2Points(_
                                  X1:=dblX, Y1:=dblY, _
                                  X2:=dblX + 0.1, Y2:=dblY)
                                                    
    ' Use the line just created to create the axis of revolution.
    Set objRefAxis = objProfile.SetAxisOfRevolution(LineForAxis:=objAxisLine)
    
    ' End and validate the profile.

    ordStatus = objProfile.End(ValidationCriteria:=igProfileClosed + igProfileRefAxisRequired)
    If ordStatus <> 0 Then
        MsgBox "Invalid Profile"
        Exit Sub
    End If
    
    ' Create either a revolved protrusion or cutout depending on how you set the case statement
    Select Case 1   '1 or 2
    Case 1
        ' Create a 180 degree revolved cutout.
        Call objModel.RevolvedCutouts.AddFinite(_
               Profile:=objProfile, RefAxis:=objRefAxis, _

               profileSide:=igLeft, ProfilePlaneSide:=igSymmetric, _
               AngleOfRevolution:=Pi)
    Case 2
        ' Create a 180 degree revolved Protrusion.
        Call objModel.RevolvedProtrusions.AddFinite(_
               Profile:=objProfile, RefAxis:=objRefAxis, _
               profileSide:=igLeft, ProfilePlaneSide:=igSymmetric, _
               AngleOfRevolution:=Pi)
    End Select
    
    ' Turn off the display of the profile.
    objProfile.Visible = False

End Sub

